// -----JS CODE-----
//@input Asset.ObjectPrefab objectPrefab

//@input float spawnFrequency{"widget":"slider","min":0.1, "max":5, "step":0.02}
//@input float spawnRandomizer{"widget":"slider","min":0, "max":0.5, "step":0.02}
//@input float spawnRange {"widget":"slider","min":0, "max":1, "step":0.1}
//@input float threshold {"widget":"slider","min":0, "max":0.5, "step":0.01}
//@input float fallingSpeedMin
//@input float fallingSpeedMax

//@input SceneObject mouthPositionObject
//@input Component.Camera camera

//@input SceneObject startButton
//@input Component.Text currentScoreNumber
//@input Component.Text missedScoreNumber

//@input SceneObject gameOverScreen
//@input float missedScoreMax

var currentScore = 0;
var nextSpawnTime =  (1/script.spawnFrequency); //reverse spawnFrequency so higher number would produce more frequent result, not necessary for our game but easier to understand.
var spawnRange = script.spawnRange;
var spawnTimer = 0;

//get screen position of this aka ObjectSpawner object
var screenTransform = script.getSceneObject().getComponent("Component.ScreenTransform");    
var myScreenPos = screenTransform.anchors.getCenter();


var missedScoreMax = script.missedScoreMax;
var missedScore = 0;

var topScore = 0;


//@input Component.Text topScoreNumber
//create reference to global persistent storage system
var store = global.persistentStorageSystem.store; 
//create a key to identify the variable to be saved
var scoreKey = "topScore";
//get from saved variable every time lens starts
topScore = store.getInt(scoreKey);
script.topScoreNumber.text = topScore.toString();

script.gameOverScreen.enabled = false;

script.createEvent("UpdateEvent").bind(function(){
    
    if(startgame){
        getMouthPosition();    

        if(spawnTimer < nextSpawnTime){
            spawnTimer += getDeltaTime();
        }else{
            spawnObject();
            spawnTimer = 0;
            nextSpawnTime = 1.0/script.spawnFrequency + (Math.random() - 0.5) * script.spawnRandomizer;
        }
        
        if(missedScore >= missedScoreMax){
            script.gameOverScreen.enabled = true;            
            print("game is over!");
            startgame = false;
        }
        
    }
});

global.behaviorSystem.addCustomTriggerResponse("START_GAME", startGame);

var startgame = false;
function startGame(){
    script.startButton.enabled = false;
    startgame = true;
}

function spawnObject(){
    //creating a copy of the prefab   
    var newObj = script.objectPrefab.instantiate(script.getSceneObject().getParent());

    //randomize position with range
    var randomXpos = myScreenPos.x + Math.random()*spawnRange*2 - spawnRange;
    var newObjPosition = new vec2(randomXpos, myScreenPos.y);
    
    //set screen position of newObj aka ObjectPrefab object
    var objScreenTransform = newObj.getComponent("Component.ScreenTransform");
    objScreenTransform.anchors.setCenter(newObjPosition);
}

function getFallingSpeed(){
    return Math.random() * (script.fallingSpeedMax - script.fallingSpeedMin) + script.fallingSpeedMin;
}

function getMouthPosition(){
    var mouthWorldPos = script.mouthPositionObject.getTransform().getWorldPosition();
    var mouthPos = script.camera.worldSpaceToScreenSpace(mouthWorldPos);
    mouthPos = new vec2(mouthPos.x*2-1, 1-mouthPos.y*2);
}

function getThreshold(){
    return script.threshold;
}

function OnHit(){
    if(startgame){
        currentScore ++;
        script.currentScoreNumber.text = currentScore.toString();
        if(currentScore > topScore){
            //update top score when current score exceeds top score
            topScore = currentScore;
            //store top score in persistent storage
            store.putInt(scoreKey, topScore);
            //update UI
            script.topScoreNumber.text = topScore.toString();
        }
    }
}


function OnMissed(){
    if(startgame){
        missedScore ++;
        script.missedScoreNumber.text = missedScore.toString();
    }
}

script.api.OnMissed = OnMissed;

script.api.OnHit = OnHit;
script.api.getMouthPosition = getMouthPosition;
script.api.getFallingSpeed = getFallingSpeed;
script.api.getThreshold = getThreshold;